/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "Application.hpp"

/**
 * Creates a callback in FreeGLUT to enter the entry point of the application.
 *
 * \param state Value is required by FreeGLUT but is ignored.
 */
static void entryCallback(int state)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onEntry();
}

/**
 * Creates a callback in FreeGLUT to enter the exit point of the application.
 */
static void closeCallback()
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onClose();

	glutLeaveMainLoop();
}

/**
 * Creates a callback in FreeGLUT to enter the display point of the application.
 */
static void displayCallback()
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onDisplay();
}

/**
 * Creates a callback in FreeGLUT to enter the idle point of the application.
 */
static void idleCallback()
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onIdle();

	glutPostRedisplay();
}

/**
 * Creates a callback in FreeGLUT to resize the window of the application.
 *
 * \param width The new width of the window
 * \param height The new height of the window
 */
static void reshapeCallback(int width, int height)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onReshape(width, height);
}

/**
 * Creates a callback in FreeGLUT for when a key is released
 *
 * \param key The key released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void keyUpCallback(unsigned char key, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onKeyUp(key, x, y);
}

/**
 * Creates a callback in FreeGLUT for when a key is pressed
 *
 * \param key The key pressed
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void keyDownCallback(unsigned char key, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onKeyDown(key, x, y);
}

/**
 * Creates a callback in FreeGLUT for when a special key is released
 *
 * \param key The key released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void specialKeyUpCallback(int key, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onSpecialKeyUp(key, x, y);
}

/**
 * Creates a callback in FreeGLUT for when a special key is pressed
 *
 * \param key The key pressed
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void specialKeyDownCallback(int key, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onSpecialKeyDown(key, x, y);
}

/**
 * Creates a callback in FreeGLUT for when the mouse is clicked
 *
 * \param button The button clicked
 * \param upOrDown Whether the button was pressed or released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void mouseClickCallback(int button, int upOrDown, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onMouseClick(button, upOrDown, x, y);
}

/**
 * Creates a callback in FreeGLUT for when the mouse wheel is moved
 *
 * \param wheel The mouse wheel moved
 * \param direction The direction it was moved
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void mouseWheelCallback(int wheel, int direction, int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onMouseWheel(wheel, direction, x, y);
}

/**
 * Creates a callback in FreeGLUT for when the mouse is held and moved
 *
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void motionCallback(int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onMotion(x, y);
}

/**
 * Creates a callback in FreeGLUT for when the mouse is moved
 *
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
static void passiveMotionCallback(int x, int y)
{
	Application* app = (Application*)Application::TheApplication;

	if (app)
		app->onPassiveMotion(x, y);
}

/**
 * The main loop of the window application.
 *
 * This function sets up all the communication between FreeGLUT and the application.
 * This function also sets up OpenGL and its extensions via GLEW.
 *
 * \param argumentCount The number of arguments
 * \param argumentValues The values of the arguments
 */
int Application::main(int argumentCount, char** argumentValues)
{
	Application* app = (Application*)Application::TheApplication;

	ilInit();
	ilEnable(IL_CONV_PAL);
	ilutEnable(ILUT_OPENGL_CONV);

	glutInit(&argumentCount, argumentValues);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STENCIL);
	glutInitWindowSize(app->getWidth(), app->getHeight());
	glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_CONTINUE_EXECUTION) ;

	app->setWindowID(glutCreateWindow(app->getWindowTitle()));

	glewInit();

	app->onEntry();

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	glutDisplayFunc(displayCallback);
	glutIdleFunc(idleCallback);
	glutReshapeFunc(reshapeCallback);
	glutKeyboardFunc(keyDownCallback);
	glutSpecialFunc(specialKeyDownCallback);
	glutKeyboardUpFunc(keyUpCallback);
	glutSpecialUpFunc(specialKeyUpCallback);
	glutMouseFunc(mouseClickCallback);
	glutMouseWheelFunc(mouseWheelCallback);
	glutMotionFunc(motionCallback);
	glutPassiveMotionFunc(passiveMotionCallback);
	glutEntryFunc(entryCallback);
	glutCloseFunc(closeCallback);

	glutMainLoop();

	return(0);
}
