/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "ProgramObject.hpp"

/**
 * Constructor for a ProgramObject
 */
ProgramObject::ProgramObject()
{
	program = glCreateProgramObjectARB();
}

/**
 * Destructor for a ProgramObject
 */
ProgramObject::~ProgramObject()
{
	glDeleteObjectARB(program);
}

/**
 * Attaches a ShaderObject to the ProgramObject
 *
 * \param shader The ShaderObject to attach
 */
void ProgramObject::addShader(ShaderObject* shader)
{
	glAttachObjectARB(program, shader->getShader());
}

/**
 * Detaches a ShaderObject from the ProgramObject
 *
 * \param shader The ShaderObject to detach
 */
void ProgramObject::removeShader(ShaderObject* shader)
{
	glDetachObjectARB(program, shader->getShader());
}

/**
 * Links the ShaderObjects to the ProgramObject
 *
 * Call this after all the ShaderObjects have been attached
 * to the ProgramObject. The ShaderObject must be linked before
 * it can be used.
 */
void ProgramObject::link()
{
	GLint linked;

	glLinkProgramARB(program);

	glGetObjectParameterivARB(program, GL_OBJECT_LINK_STATUS_ARB, &linked);

	assert(linked && "There was an error while linking the shader object");
}

/**
 * Begins the ProgramObject effect
 */
void ProgramObject::begin()
{
	glUseProgramObjectARB(program);
}

/**
 * Ends the ProgramObject effect
 */
void ProgramObject::end()
{
	glUseProgramObjectARB(0);
}

/**
 * Sets a single float uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The value to set the uniform to
 */
void ProgramObject::setUniform1f(const char* varname, GLfloat v0)
{
	glUniform1fARB(getUniformLocation(varname), v0);
}

/**
 * Sets a 2 float uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 */
void ProgramObject::setUniform2f(const char* varname, GLfloat v0, GLfloat v1)
{
	glUniform2fARB(getUniformLocation(varname), v0, v1);
}

/**
 * Sets a 3 float uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 * \param v2 The third value to set the uniform to
 */
void ProgramObject::setUniform3f(const char* varname, GLfloat v0, GLfloat v1, GLfloat v2)
{
	glUniform3fARB(getUniformLocation(varname), v0, v1, v2);
}

/**
 * Sets a 4 float uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 * \param v2 The third value to set the uniform to
 * \param v3 The fourth value to set the uniform to
 */
void ProgramObject::setUniform4f(const char* varname, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
{
	glUniform4fARB(getUniformLocation(varname), v0, v1, v2, v3);
}

/**
 * Sets a 1 int uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The value to set the uniform to
 */
void ProgramObject::setUniform1i(const char* varname, GLint v0)
{
	glUniform1iARB(getUniformLocation(varname), v0);
}

/**
 * Sets a 2 int uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 */
void ProgramObject::setUniform2i(const char* varname, GLint v0, GLint v1)
{
	glUniform2iARB(getUniformLocation(varname), v0, v1);
}

/**
 * Sets a 3 int uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 * \param v2 The third value to set the uniform to
 */
void ProgramObject::setUniform3i(const char* varname, GLint v0, GLint v1, GLint v2)
{
	glUniform3iARB(getUniformLocation(varname), v0, v1, v2);
}

/**
 * Sets a 4 int uniform variable
 *
 * \param varname The name of the uniform
 * \param v0 The first value to set the uniform to
 * \param v1 The second value to set the uniform to
 * \param v2 The third value to set the uniform to
 * \param v3 The fourth value to set the uniform to
 */
void ProgramObject::setUniform4i(const char* varname, GLint v0, GLint v1, GLint v2, GLint v3)
{
	glUniform4iARB(getUniformLocation(varname), v0, v1, v2, v3);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform1fv(const char* varname, GLsizei count, GLfloat *value)
{
	glUniform1fv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform2fv(const char* varname, GLsizei count, GLfloat *value)
{
	glUniform2fv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform3fv(const char* varname, GLsizei count, GLfloat *value)
{
	glUniform3fv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform4fv(const char* varname, GLsizei count, GLfloat *value)
{
	glUniform4fv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform1iv(const char* varname, GLsizei count, GLint *value)
{
	glUniform1iv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform2iv(const char* varname, GLsizei count, GLint *value)
{
	glUniform2iv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform3iv(const char* varname, GLsizei count, GLint *value)
{
	glUniform3iv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 * \param count The number of uniforms in the array
 * \param value The array of uniforms
 */
void ProgramObject::setUniform4iv(const char* varname, GLsizei count, GLint *value)
{
	glUniform4iv(getUniformLocation(varname), count, value);
}

/**
 *
 * \param varname The name of the uniform
 */
void ProgramObject::setUniformMatrix2fv(const char* varname, GLsizei count, GLboolean transpose, GLfloat *value)
{

}

/**
 *
 * \param varname The name of the uniform
 */
void ProgramObject::setUniformMatrix3fv(const char* varname, GLsizei count, GLboolean transpose, GLfloat *value)
{

}

/**
 *
 * \param varname The name of the uniform
 */
void ProgramObject::setUniformMatrix4fv(const char* varname, GLsizei count, GLboolean transpose, GLfloat *value)
{

}

/**
 *
 * \param index The index of the vertex attribute
 * \param v0 The value to set the attribute to
 */
void ProgramObject::setVertexAttrib1f(GLuint index, GLfloat v0)
{

}

/**
 *
 * \param index The index of the vertex attribute
 * \param v0 The first value to set the attribute to
 * \param v1 The second value to set the attribute to
 */
void ProgramObject::setVertexAttrib2f(GLuint index, GLfloat v0, GLfloat v1)
{

}

/**
 *
 * \param index The index of the vertex attribute
 * \param v0 The first value to set the attribute to
 * \param v1 The second value to set the attribute to
 * \param v2 The third value to set the attribute to
 */
void ProgramObject::setVertexAttrib3f(GLuint index, GLfloat v0, GLfloat v1, GLfloat v2)
{

}

/**
 *
 * \param index The index of the vertex attribute
 * \param v0 The first value to set the attribute to
 * \param v1 The second value to set the attribute to
 * \param v2 The third value to set the attribute to
 * \param v3 The fourth value to set the attribute to
 */
void ProgramObject::setVertexAttrib4f(GLuint index, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
{

}

/**
 * Retrieves the location of a uniform variable
 *
 * \param name The name of the uniform
 * \returns The location of the uniform
 */
GLint ProgramObject::getUniformLocation(const char* name)
{
	GLint location;

	location = glGetUniformLocationARB(program, name);

	assert(location != -1 && "Uniform variable not found");

	return(location);
}
